@@ -3,7 +3,7 @@ package Tie::REHash;
 use 5.006; 
 
 use strict qw[vars subs];
-$Tie::REHash::VERSION = '1.04_03'; 
+$Tie::REHash::VERSION = '1.01';
 
 no warnings; 
 
@@ -12,7 +12,6 @@ sub CMIS () { 1 }
 sub CHIT () { 1 } 
 sub OFFSET () { 0 } 
 our (%Global_options, %AD);
-our $qr_fragment = qr{(subcall[\w\d]+)(?:\(\))?}; 
 
 $AD{croak} = 'use Carp;';
 
@@ -200,47 +199,6 @@ sub store {
 }
 SUBCODE
 
-$AD{subcall_match} = <<'SUBCODE';
-SUBCODE
-$AD{subcall_match2} = <<'SUBCODE';
-	!$cach2
-	? $self->{REGX2}{ $_}
-	: ( CDUP 
-	? $self->{REGX}[0]{$k}
-	: ${ $cach2->{$k}
-	 = \$self->{REGX2}{ $_} } )
-SUBCODE
-$AD{subcall_match0} = <<'SUBCODE';
-	!$cach2
-	? undef
-	: ( CDUP 
-	? $self->{REGX}[0]{$k}
-	: $cach2->{$k} = undef
-	, $self->{ESC}{$k} = 1 )[0] 
-SUBCODE
-
-$AD{subcall_fetch} = <<'SUBCODE';
-	($_ eq $cach ? ${$_->{$k}} 
-	 : $_->{$k})
-SUBCODE
-$AD{subcall_fetch2} = <<'SUBCODE';
-	!$cach2
-	? $_ eq $cach ? ${$_->{$k}} 
-	 : $_->{$k}
-	: ( CDUP 
-	? $self->{REGX}[0]{$k}
-	: ${ $cach2->{$k}
-	 = \$_->{$k} } ) 
-SUBCODE
-$AD{subcall_fetch0} = <<'SUBCODE';
-	!$cach2
-	? undef
-	: ( CDUP 
-	? $self->{REGX}[0]{$k}
-	: $cach2->{$k}
-	 = undef )
-SUBCODE
-
 $AD{fetch} = <<'SUBCODE';
 sub fetch { 
 	my $self = $_[0];
@@ -291,24 +249,41 @@ sub fetch {
 			if ( CHIT 
 			and $self->{CHIT2}
 			) {
+
 				CDUP
 				and ref $self->{REGX}[0] eq 'HASH'
 				|| unshift @{$self->{REGX}}, {};
 
 				return 
 				 exists $self->{ESC2}{$_}
-				? subcall_match0()
+				? ( !$cach2
+				? undef
+				: ( CDUP 
+				? $self->{REGX}[0]{$k}
+				: $cach2->{$k} = undef
+				, $self->{ESC}{$k} = 1 )[0] )
 				: $type eq 'ex'
 				? 1
 				: exists $self->{DYN2}{$_} && !$esc 
 				? $type eq 'sr' 
-				? \(subcall_match2()->($k)) 
-				: subcall_match2()->($k)
+				? \$self->{REGX2}{ $_}($k) 
+				: $self->{REGX2}{ $_}($k)
 				: $type eq 'sr' 
-				? \(subcall_match2())
-				: subcall_match2();
+				? \( !$cach2
+				? $self->{REGX2}{ $_}
+				: ( CDUP 
+				? $self->{REGX}[0]{$k}
+				: ${ $cach2->{$k}
+				 = \$self->{REGX2}{ $_}} ) )
+				: ( !$cach2
+				? $self->{REGX2}{ $_}
+				: ( CDUP 
+				? $self->{REGX}[0]{$k}
+				: ${ $cach2->{$k}
+				 = \$self->{REGX2}{ $_} } ) );
 			}
 			else {
+
 				return 
 				 exists $self->{ESC2}{$_}
 				? undef
@@ -329,24 +304,43 @@ sub fetch {
 			if ( CHIT 
 			and $self->{CHIT2}
 			) {
+
 				CDUP
 				and ref $self->{REGX}[0] eq 'HASH'
 				|| unshift @{$self->{REGX}}, {};
 
 				return 
 				 exists $self->{ESC}{$k}
-				? subcall_fetch0() 
+				? ( !$cach2
+				? undef
+				: ( CDUP 
+				? $self->{REGX}[0]{$k}
+				: $cach2->{$k}
+				 = undef ) ) 
 				: $type eq 'ex'
 				? 1
 				: exists $self->{DYN}{$k} && !$esc 
 				? $type eq 'sr' 
-				? \(subcall_fetch2()->($k)) 
-				: subcall_fetch2()->($k) 
+				? \$_->{$k}($k) 
+				: $_->{$k}($k) 
 				: $type eq 'sr' 
-				? \(subcall_fetch2()) 
-				: subcall_fetch2(); 
+				? \( !$cach2
+				? $_ eq $cach ? ${$_->{$k}} 
+				 : $_->{$k}
+				: ( CDUP 
+				? $self->{REGX}[0]{$k}
+				: ${ $cach2->{$k}
+				 = \$_->{$k} } ) )
+				: ( !$cach2
+				? $_ eq $cach ? ${$_->{$k}} 
+				 : $_->{$k}
+				: ( CDUP 
+				? $self->{REGX}[0]{$k}
+				: ${ $cach2->{$k}
+				 = \$_->{$k} } ) ); 
 			}
 			else {	
+
 				return 
 				 exists $self->{ESC}{$k}
 				? undef
@@ -354,11 +348,13 @@ sub fetch {
 				? 1
 				: exists $self->{DYN}{$k} && !$esc 
 				? $type eq 'sr' 
-				? \(subcall_fetch()->($k)) 
-				: subcall_fetch()->($k) 
+				? \$_->{$k}($k) 
+				: $_->{$k}($k) 
 				: $type eq 'sr' 
-				? \subcall_fetch()
-				: subcall_fetch();
+				? \($_ eq $cach ? ${$_->{$k}} 
+				 : $_->{$k})
+				: ($_ eq $cach ? ${$_->{$k}} 
+				 : $_->{$k});
 
 			}
 		} 
@@ -716,9 +712,11 @@ sub clear {
 }
 SUBCODE
 
+$AD{DESTROY} = <<'SUBCODE';
 sub DESTROY {
 	%{$_[0]} = ();
 }
+SUBCODE
 
 *storable = \&freeze;
 $AD{freeze} = <<'SUBCODE';
@@ -864,7 +862,7 @@ SUBCODE
 
 $AD{die_on_stringref_bug} = $AD{stringref_bug_is_fixed} = <<'SUBCODE';
 sub die_on_stringref_bug {
-	my $bugtxt = 'Due to bug (rt.perl.org ticket 79178) in your instance of perl, storing/fetching to/from the rehash should avoid escaped literal keys (as well as stringified scalarref keys), like $hash{\"foo"} (or in one statement: $regx2{$k = \"foo"}), or fatal error will result. The workaround: $k = \"foo"; $hash{$k}.';
+	my $bugtxt = 'Due to bug (rt.perl.org ticket 79178) introduced in perl v5.12.0 and persisting at least up to v5.12.2, storing/fetching to/from the rehash should avoid escaped literal keys (as well as stringified scalarref keys), like $hash{\"foo"}, or fatal error will result. The workaround: $k = \"foo"; $hash{$k}.';
 	#warn("BUG WARNING: $bugtxt"); 
 
 	*FETCH2 = \&FETCH;
@@ -897,13 +895,9 @@ SUBCODE
 
 $AD{precompile} = <<'SUBCODE';
 sub precompile {
-	foreach (keys %AD) {
-		next 
-		if $_ =~ $qr_fragment; 
-		 $AD{$_} =~ s/$qr_fragment/($AD{$1})/g;
-		eval $AD{$_};
-		!$@ or croak( "Compilation error: $@ in code: $AD{$_}")
-	}
+	eval( $AD{$_}), !$@ || carp( "Compilation error: "
+	. "$@ in code: $AD{$_}")
+	foreach keys %AD;
 
 	return 1
 }
@@ -917,12 +911,8 @@ sub is_precompiled { 1 }
 SUBCODE
 
 sub AUTOLOAD {
-	my $code = \( $AD{$Tie::REHash::AUTOLOAD}
-	 || scalar( $Tie::REHash::AUTOLOAD =~ /::(\w+)$/, $AD{$1} ) );
-	eval{ $$code =~ s/$qr_fragment/($AD{$1})/g; }; 
-	eval $$code; 
-	!$@ or die("$@ evaluating $$code");
-
+	eval( $AD{$Tie::REHash::AUTOLOAD}
+	 ||( $Tie::REHash::AUTOLOAD =~ /::(\w+)$/, $AD{$1} ) );
 	goto &$Tie::REHash::AUTOLOAD 
 	if defined &$Tie::REHash::AUTOLOAD;
 }
@@ -977,19 +967,13 @@ Tie::REHash - the tie()d implementation of RegExp Hash (REHash) that allows usin
 
 Tie::REHash is a tie()d implementation of hash that allows using regexp "keys" along with plain keys.
 
-Storing regexp key in a hash tie()d to Tie::REHash is equivalent to storing set of (plain) string keys that regexp matches, called "matching keys". Matching key, as well as regexp key that created it, exists(). Regexp key and all its matching keys share same value. If we take dictionary view of a hash, then Tie::REHash implementation of hash allows using regexp keys to effectively define (patterned) sets of synonymous (aliased) keys.
+Storing regexp key in a hash tie()d to Tie::REHash is equivalent to storing set of string keys that regexp matches, called "matching keys". Matching key (as well as regexp key that created it) exists(). Regexp key and all its matching keys share same single value. If we take dictionary view of a hash (in some programming languages they call "dictionary" what we call "hash"), then Tie::REHash is the implementation of hash that effectively allows defining arbitrary sets of synonymous (aliased) keys by using regexp keys for that.
 
-To make matters worse, Tie::REHash also supports notion of "dynamic value" (this extention can safely be ignored, if it is not necessary).
+To make matters worse, Tie::REHash also supports notion of "dynamic value".
 
-As a result of these enhancements, Tie::REHash allows, for example, creating hash with infinite sets of key/value pairs.
+As a result of these enhancements, Tie::REHash allows, for example, creating hash with infinite sets of key/value pairs. 
 
-In this documentation name "REHash" is often used as short reference to Tie::REHash class or object, while lowercased "rehash" term is used to refer to hash tie()d to Tie::REHash, i.e. the regexp keys hash, to distinguish it from standard hash (however, since rehash behaves almost like standard hash, rehash may also be referred to as simply "hash" in the standard hash context).
-
-=head1 How to read this documentation
-
-This documentation covers many additional features (e.g. dynamic values, performnce tuning, serialization, etc.) that you do NOT need to learn for using Tie::REHash in most usual case. Thus, this documentation can be read selectively, safely skipping most of it, as follows:
-
-First read SYNOPSIS. Then you may read only those sections that concern your usage. This way you can easily avoid learning features that you do not need. Initially, you may read only code examples, skipping text, and read text only if necessary (as a last resort). Later alows to avoid documentation verbosity, unless verbosity is what you need. 
+In this documentation name "REHash" is often used as short reference to Tie::REHash class or object, while lowercased "rehash" is used to refer to hash tie()d to Tie::REHash, the regexp keys hash, to distinguish it from standard hash (however, since rehash behaves almost like standard hash, rehash may also be referred to as simply "hash" in the standard hash context).
 
 =head1 Rehash construction
 
@@ -1051,7 +1035,7 @@ Storing regexp key in rehash effectively stores a set of string keys that regexp
 	exists $rehash{automobile};                         # true
 	exists $rehash{qr{car|automobile}};                 # true
 
-Thus, keys of rehash are divided into two main classes: string keys and regexp keys. Storing regexp key creates coresponding string keys - matching keys. So, string keys are in turn can be divided into matching keys and plain keys. Keys of different classes, however, behave identically, except matching keys are not returned by keys(), values(), each() and when evaluating hash in list context (see corresponding section below). 
+Keys of rehash are divided into two main classes: string keys and regexp keys. String keys are in turn divided into matching keys and plain keys. Keys of different classes, however, behave identically, except matching keys are not returned by keys(), values(), each() and when evaluating hash in list context (see corresponding section below). 
 
 Value of specific matching key created by one regexp key is overridden by stored later either same plain key or other regexp key that also matches that same key:
 
@@ -1064,7 +1048,7 @@ Value of specific matching key created by one regexp key is overridden by stored
 	exists $rehash{qr{car|automobile}}                         # true
 	and    $rehash{qr{car|automobile}} eq 'vehicle on wheels'; # true
 
-Each individual matching key exists() and can be delete()d, but it is not returned by keys() and each() (see below); later is the only difference between matching key and plain key. 
+Matching key do exists() and can be delete()d, but it is not returned by keys() and each() (see below); later is the only difference between matching key and plain key. 
 
 Accordingly, in case of rehash each() key returned by keys() exists(), but, unlike in case of standard hash, the reverse is not true - matching key exists(), but is not returned by keys() and each(). (For more details refer to section "keys(), values(), each() and List Context" below.)
 
@@ -1103,8 +1087,6 @@ Regexps that are equivalent in terms of what they match, but written differently
 
 =head2 Dynamic Values
 
-Dynamic values feature of rehash can safely be ignored skipping this documentation section entirely, if you do not plan to use it.
-
 Dynamic value of the key is simply reference to subroutine (called "dynamic value subroutine") that is called when key's value is fetched, with accessed key passed as argument; the value returned by that subroutine (called "calculated value") is then returned as key's value.
 
 In contrast to dynamic and calculated values, plain values are often called "static" (or "non-dynamic").
@@ -1118,7 +1100,7 @@ Both plain and regexp/matching keys may have dynamic values. In case of regexp/m
 After being stored, dynamic value subroutine is revealed instead of its calculated values only in the following cases: 
 
 1) reference to dynamic value subroutine is returned by values() and each(), together with corresponding escaped key (returned by keys() in case of values()); 
-2) reference to dynamic value subroutine is fetched using escaped key (i.e. both key and value are same that were stored - see Escaped Keys). 
+2) reference to dynamic value subroutine is fetched as value of escaped key (i.e. both key and value are same that were stored - see Escaped Keys). 
 
 In all other cases dynamic value is represented by its calculated value and indistinguishable from static value (except calculated value may change). 
 
@@ -1140,7 +1122,7 @@ Since dynamic value of the key gets calculated, it can vary. In particular, if r
 
 Moreover, using regexp key one may not only create infinite set of keys, but also use dynamic value to create corresponding infinite set of values. For example, the following rehash is set to have an infinite set of key/value pairs.
 
-	$rehash{\qr{.*}} = sub { $_[0] }; 
+	$rehash{\qr{.*}} = sub { $1 }; 
 
 =head2 Escaped Keys
 
@@ -1237,11 +1219,11 @@ Unlike string keys, fetching regexp keys is always fast (but those are unlikely
 
 Evaluating rehash in list context, as well as calling keys(), values(), first call of each() or evaluating rehash in list context all may be relatively costly in case of rehash with large number of plain/regexp keys. In this case or in case of tight loop, the tied(%rehash)->keys(), tied(%rehash)->values() or tied(%rehash)->list() are better used instead of, respectively, keys(%rehash), values(%rehash) or evaluating %rehash in list context, since these methods can be times (5-6 in some benchmarks) faster. See "keys(), values() and list() methods" section below. 
 
-Note also that mere use()/require()ing of Tie::REHash is relatively inexpensive. For comparison, it is about twice as costly as use Carp (faster CPU make this ratio even closer to 1), so that in most cases there is no need to be specifically conserned about use()/require()ing it unnecessarily (like in case of Carp itself).
+Note also that mere use()/require()ing of Tie::REHash is relatively inexpensive. For comparison, it is about twice as costly as use Carp, so that in most cases there is no need to be specifically conserned about use()/require()ing it unnecessarily (like in case of Carp itself).
 
 =head2 Caching 
 
-Rehash has built-in cache to improve performance of repeated same key hits and misses. The caching is off by default - to enable it for specific rehash set do_cache() attribute true:
+Rehash has built-in cache to improve performance of repeated same key hits and misses. The caching is off by default - to enable it for specific hash set do_cache() attribute true:
 
 	tied(%rehash)->do_cache(1) 
 
@@ -1249,21 +1231,21 @@ If do_cache() attribute is true, additional attributes do_cache_hit() and do_cac
 
 The approximate rule is: if number of regexp keys in the hash is equal or higher than the value of do_cache_miss() attribute (i.e. the miss is costly enough), the caching of misses is on - repeated same key misses are fast. False do_cache_miss() turns caching of misses off. The default is true do_cache_miss(1).
 
-If do_cache_hit() attribute is set true, string key hits are cached - repeated same key hits are fast. False do_cache_hit() turns caching of hits off. The default is true do_cache_hit(1).
+If do_cache_hit() attribute is set true, string key hits (except dynamic values) are cached - repeated same key hits are fast. False do_cache_hit() turns caching of hits off. The default is true do_cache_hit(1).
 
-Caching pays off only if repeated fetches of same key happen often enough; otherwise caching just adds overhead without actually using the cache very much. Moreover, caching is useless if rehash has no regexp keys (and the more regexp keys are in the rehash, the more efficiency benefits caching can bring). For that reason, caching is off by default and should be turned on manually only for those rehashes that need it. Alternatively, caching can be turned on by default for all rehashes upon Tie::REHash loading:
+Caching pays off only if repeated fetches of same key happen often enough; otherwise caching just adds overhead without actually using the cache very much. For that reason, caching is off by default and should be turned on manually only for those rehashes that need it. Alternatively, caching can be turned on by default for all hashes upon Tie::REHash loading:
 
 	use Tie::REHash do_cache => 1;
 
-Performance of repeated fetching of dynamic value also improves with caching (same way as that of plain value), but dynamic value subroutine is still called every time, i.e. dynamic value may change upon repeated fetch.
+Dynamic values are never cached, since they are dynamic by nature and may change upon repeated fetch.
 
-Use of caching may dramatically improve performance of repeated fetches. For example, hash with 100 simple regexp keys may get up to 30 times boost in speed of repeated fetches. Also note that caching do not copy rehash values, so the memory footprint do not escalate as a result of caching.
+Use of caching may dramatically improve performance of repeated fetches. For example, hash with 100 simple regexp keys may get up to 30 times boost in speed of repeated fetches. Also note that cache do not copy rehash values, so the memory footprint do not escalate as a result of caching.
 
 To empty cache of specific hash, call tied(%rehash)->flush_cache().
 
 =head1 Serialization
 
-The generic rehash serialization/deserialization sequence that works for most serializers is as follows (where serialize() and deserialize() stand for corresponding routines provided by a serializer module):
+The generic rehash serialization/deserialization sequence that works for most serializers is as follows:
 
 	$data = serialize(tied(%rehash)->freeze);
 	tie %clone, Tie::REHash->unfreeze(deserialize($data));
@@ -1336,16 +1318,16 @@ However, if called in scalar context with true argument, the return value is ref
 
 =head2 freeze() (aliase: storable())
 
-	$data   = tied(%rehash)->freeze();       # same as freeze('data')
-	$data   = tied(%rehash)->freeze('data'); # same as freeze()
+	$data   = tied(%rehash)->freeze();
+	$data   = tied(%rehash)->freeze('data');
 	$object = tied(%rehash)->freeze('clone');
 	$object = tied(%rehash)->freeze('itself');
 
 The freeze() method always returns serializable (frozen) data structure that is a "snapshot" of Tie::REHash object instance that freeze() was called on. The returned snapshot data structure can then be serialized using some serializer. The type of data structure depends on argument, as follows:
 
-'data'   - (default assumed if not argument is specified) unblessed data structure copied from Tie::REHash instance (later is not altered);
+'data'   - unblessed data structure copied from Tie::REHash instance (later is not altered);
 'clone'  - blessed copy of Tie::REHash instance (later is not altered);
-'itself' - the very Tie::REHash instance that freeze() was called on is converted into serializable, but non-operational (so rehash becomes non-operational!) state and returned.
+'itself' - the very Tie::REHash instance that freeze() was called on is converted into serializable, but non-operational (!), state and returned.
 
 If no or some other argument is specified, freeze() defaults to 'data' mode. With both 'data' and 'clone' arguments freeze() returns shallow copies that share most of its data with Tie::REHash instance - those data structures should be used in read-only mode.
 
@@ -1412,9 +1394,7 @@ The optimal value of autodelete_limit() is approximated by the number of plain k
 
 =head1 BUGS
 
-Due to bug (rt.perl.org ticket 79178) in some perls (not bound to specific versions range), storing/fetching to/from the rehash should avoid escaped literal keys (as well as stringified scalarref keys), like $rehash{\"foo"}, or fatal error will result. The workaround: $key = \"foo"; $rehash{$key} (or in one statement: $rehash{$key = \"foo"}). If your perl is affected, you will see BUG WARNING during module installation. 
-
-Due to incomplete implementation of hash tie()ing in perls prior to v5.8.3, evaluating hash (tie()d to Tie::REHash) in scalar context will not work as expected - use tied(%hash)->scalar instead.
+Due to bug (rt.perl.org ticket 79178) introduced in perl v5.12.0 and persisting at least up to v5.12.2 (reportedly fixed in v5.13 and newer), storing/fetching to/from the rehash should avoid escaped literal keys (as well as stringified scalarref keys), like $rehash{\"foo"}, or fatal error will result. The workaround: $key = \"foo"; $rehash{$key}.
 
 =head1 TODO
 
